/**
 *   Copyright:
 *   Copyright (C) 2010 by Greenbone Networks GmbH
 *
 *   Authors:
 *   Raimund Renkert <raimund.renkert@greenbone.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "gsd_dlg.h"
#include "gsd_control.h"

/**
 * @file gsd_dlg.cpp
 * @brief Holds the dialogs.
 */

/**
 * @brief Performs initial actions to control the gsd dialogs.
 *
 * @param[in]  ctl    GSD-Controller.
 */
gsd_dlg::gsd_dlg (gsd_control *ctl)
{
  this->control = ctl;

  about = new dlg_about ();
  login = new dlg_login ();
  start_gsa = new dlg_start_gsa ();
  new_task = new dlg_new_task ();
  new_target = new dlg_new_target ();
  new_config = new dlg_new_config ();
  new_schedule = new dlg_new_schedule ();
  new_escalator = new dlg_new_escalator ();
  new_credential = new dlg_new_credential ();
  new_agent = new dlg_new_agent ();
  new_slave = new dlg_new_slave ();

  connect (login,
           SIGNAL (sig_save_profile (QString, omp_credentials&)),
           control,
           SLOT (profile_save (QString, omp_credentials&)));
  connect (login,
           SIGNAL (sig_login (QString, int, QString, QString)),
           this,
           SLOT (accept_login (QString, int, QString, QString)));
  connect (login,
           SIGNAL (sig_check_server (QString, int, QString, QString)),
           this,
           SLOT (check_server (QString, int, QString, QString)));
  connect (login,
           SIGNAL (sig_profile_selected (int)),
           this,
           SLOT (login_update (int)));
  connect (login,
           SIGNAL (sig_remove_profile (int)),
           control,
           SLOT (profile_remove (int)));
  connect (new_task,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_target,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_config,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_schedule,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_escalator,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_credential,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_agent,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap <QString, QString>)));
  connect (new_slave,
           SIGNAL (sig_create (int, QMap<QString, QString>)),
           this,
           SLOT (create (int, QMap<QString, QString>)));
}

gsd_dlg::~gsd_dlg ()
{
  free (about);
  free (login);
  free (start_gsa);

  free (new_task);
  free (new_target);
  free (new_config);
  free (new_schedule);
  free (new_escalator);
  free (new_credential);
  free (new_agent);
}


/**
 * @brief Receives create signal send by dialogs and forwards it to the
 *        controller.
 *
 * @param[in]  type       Entity type.
 * @param[in]  parameter  Parameter for creation process.
 */
void
gsd_dlg::create (int type, QMap<QString, QString> parameter)
{
  emit sig_create (type, parameter);
}


/**
 * @brief Shows the new agent dialog.
 */
void
gsd_dlg::new_agent_dlg ()
{
  new_agent->show ();
}


/**
 * @brief Shows the new config dialog.
 */
void
gsd_dlg::new_config_dlg ()
{
  new_config->show ();
}


/**
 * @brief Shows the new credential dialog.
 */
void
gsd_dlg::new_credential_dlg ()
{
  new_credential->show ();
}


/**
 * @brief Shows the new escalator dialog.
 */
void
gsd_dlg::new_escalator_dlg ()
{
  new_escalator->show ();
}


/**
 * @brief Shows the new schedule dialog.
 */
void
gsd_dlg::new_schedule_dlg ()
{
  new_schedule->show ();
}


/**
 * @brief Prepares and shows the new target dialog.
 */
void
gsd_dlg::new_target_dlg ()
{
  model_omp_entity *credential = control->getCredentialModel ();

  new_target->load ();

  int i = 0;
  new_target->addCredential ("--", "--");
  while (i < credential->rowCount ())
    {
      QDomElement elem = credential->getEntity (i);
      QString name = credential->getValue (elem, "name");
      QString id = credential->getAttr (elem, "target id");
      new_target->addCredential (name, id);
      i++;
    }

  new_target->show ();
}


/**
 * @brief Prepares an shows the new task dialog.
 */
void
gsd_dlg::new_task_dlg ()
{
  model_omp_entity *config = control->getConfigModel ();
  model_omp_entity *target = control->getTargetModel ();
  model_omp_entity *schedule = control->getScheduleModel ();
  model_omp_entity *escalator = control->getEscalatorModel ();
  model_omp_entity *slave = control->getSlaveModel ();
  new_task->load ();

  int i = 0;
  while (i < config->rowCount ())
    {
      QDomElement elem = config->getEntity (i);
      QString name = config->getValue (elem, "name");
      QString id = config->getAttr (elem, "config id");
      new_task->addConfig (name, id);
      i++;
    }
  i = 0;
  while (i < target->rowCount ())
    {
      QDomElement elem = target->getEntity (i);
      QString name = target->getValue (elem, "name");
      QString id = target->getAttr (elem, "target id");
      new_task->addTarget (name, id);
      i++;
    }
  i = 0;
  new_task->addSchedule ("--", "--");
  while (i < schedule->rowCount ())
    {
      QDomElement elem = schedule->getEntity (i);
      QString name = schedule->getValue (elem, "name");
      QString id = schedule->getAttr (elem, "schedule id");
      new_task->addSchedule (name, id);
      i++;
    }
  i = 0;
  new_task->addEscalator ("--", "--");
  while (i < escalator->rowCount ())
    {
      QDomElement elem = escalator->getEntity (i);
      QString name = escalator->getValue (elem, "name");
      QString id = escalator->getAttr (elem, "escalator id");
      new_task->addEscalator (name, id);
      i++;
    }
  i = 0;
  new_task->addSlave ("--", "--");
  while (i < slave->rowCount ())
    {
      QDomElement elem = slave->getEntity (i);
      QString name = slave->getValue (elem, "name");
      QString id = slave->getAttr (elem, "slave id");
      new_task->addSlave (name, id);
      i++;
    }

  new_task->show ();
}


void
gsd_dlg::new_slave_dlg ()
{
  new_slave->show ();
}

void
gsd_dlg::request_failed (int type, int command, int err)
{}


/**
 * @brief Shows the about dialog.
 */
void
gsd_dlg::about_dlg ()
{
  this->about->load ();
  this->about->show ();
}


/**
 * @brief Shows the start-gsa dialog.
 */
void
gsd_dlg::start_gsa_dlg ()
{
  this->start_gsa->setDefaultURL (this->control->getLoginData ()->getServerAddress ());
  this->start_gsa->load ();
  this->start_gsa->show ();
}


/**
 * @brief Prepares and shows the login dialog.
 */
void
gsd_dlg::login_dlg ()
{
  for (int i = 0; i < control->getProfileCount (); i++)
    {
      login->setProfile (control->getProfile (i), i);
    }
  QSettings settings ("Greenbone", "GSD");
  settings.beginGroup ("Login");
  login->selectProfile (settings.value ("profile").toInt ());
  settings.endGroup ();
  this->login->show ();
  login->checkServer ();
}


/**
 * @brief Updates the login dialog content.
 *
 * @param[in]  index   Profile index.
 */
void
gsd_dlg::login_update (int index)
{
  QString name = control->getProfile (index);
  omp_credentials *crd = control->getProfile (name);

  login->setAddress (crd->getServerAddress ());
  login->setPort (crd->getServerPort ());
  login->setUsername (crd->getUserName ());
  login->checkServer ();
}


/**
 * @brief Receives the login data and forwards the data to the controller.
 *
 * @param[in]  address    Serveradress.
 * @param[in]  port       Serverport.
 * @param[in]  user       Username.
 * @param[in]  password   Userpassword.
 */
void
gsd_dlg::accept_login (QString address,
                               int port,
                               QString user,
                               QString password)
{
  emit sig_login (address, port, user, password);
}


void
gsd_dlg::check_server (QString address,
                       int port,
                       QString user,
                       QString password)
{
  emit sig_check_server (address, port, user, password);
}

/**
 * @brief Closes the login dialog.
 */
void
gsd_dlg::authentication_successful ()
{
  login->close ();
}


void
gsd_dlg::check_result (int res)
{
  login->setUrlValid (res);
}


bool
gsd_dlg::loginOpen ()
{
  return login->isOpen ();
}

